@@ -7,7 +7,7 @@ use strict;
 use warnings;
 use FindBin;
 use File::Spec     ();
-use File::ShareDir ();
+use File::ShareDir::PAR ();
 
 # Load every exportable constant into here, so that they come into
 # existance in the Wx:: packages, allowing everywhere else in the code to
@@ -15,7 +15,7 @@ use File::ShareDir ();
 use Wx        ':everything';
 use Wx::Event ':everything';
 
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 
 
 
@@ -27,7 +27,7 @@ our $VERSION = '0.14';
 sub share () {
 	return File::Spec->catdir( $FindBin::Bin, File::Spec->updir, 'share' ) if $ENV{PADRE_DEV};
 	return File::Spec->catdir( $ENV{PADRE_PAR_PATH}, 'inc', 'share' )      if $ENV{PADRE_PAR_PATH};
-	return File::ShareDir::dist_dir('Padre');
+	return File::ShareDir::PAR::dist_dir('Padre');
 }
 
 sub sharedir {
@@ -51,6 +51,12 @@ sub bitmap {
 		Wx::wxBITMAP_TYPE_XPM,
 	);
 }
+sub tango {
+	Wx::Bitmap->new(
+		sharefile( 'tango', '16x16', $_[0] ),
+		Wx::wxBITMAP_TYPE_PNG,
+	);
+}
 
 sub icon {
 	Wx::Icon->new(
@@ -60,3 +66,8 @@ sub icon {
 }
 
 1;
+
+# Copyright 2008 Gabor Szabo.
+# LICENSE
+# This program is free software; you can redistribute it and/or
+# modify it under the same terms as Perl 5 itself.
